@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2006 Sun Microsystems Inc.
+# Copyright 2006-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -17,10 +17,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-__id__        = "$Id: J2SE-access-bridge.py 2223 2007-03-28 17:06:38Z wwalker $"
-__version__   = "$Revision: 2223 $"
-__date__      = "$Date: 2007-03-28 13:06:38 -0400 (Wed, 28 Mar 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: J2SE-access-bridge.py 2361 2007-05-09 01:20:06Z lmonsanto $"
+__version__   = "$Revision: 2361 $"
+__date__      = "$Date: 2007-05-08 21:20:06 -0400 (Tue, 08 May 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import atspi
@@ -48,7 +48,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
     """
     def __init__(self, script):
         speechgenerator.SpeechGenerator.__init__(self, script)
-        
+
     def _getSpeechForLabel(self, obj, already_focused):
         """Get the speech for a label.
 
@@ -99,8 +99,18 @@ class Script(default.Script):
         Arguments:
         - app: the application to create a script for.
         """
+
+        # Set the debug level for all the methods in this script.
+        #
+        self.debugLevel = debug.LEVEL_FINEST
+
         default.Script.__init__(self, app)
 
+    def _debug(self, msg):
+        """ Convenience method for printing debug messages
+        """
+        debug.println(self.debugLevel, "J2SE-access-bridge.py: "+msg)
+
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
@@ -115,7 +125,7 @@ class Script(default.Script):
         Returns True if the event is of interest.
         """
         keysym = keyboardEvent.event_string
-        keyboardEvent.hw_code = keybindings._getKeycode(keysym)
+        keyboardEvent.hw_code = keybindings.getKeycode(keysym)
         return default.Script.consumesKeyboardEvent(self, keyboardEvent)
 
     def onFocus(self, event):
@@ -125,29 +135,32 @@ class Script(default.Script):
         - event: the Event
         """
 
+        self._debug("onFocus: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
         role = event.source.role
         if role == rolenames.ROLE_LIST:
-	    selectedItem = None
-	    selection = event.source.selection
-	    if selection and selection.nSelectedChildren > 0:
-        	selectedItem = atspi.Accessible.makeAccessible(
-            				selection.getSelectedChild(0))
-	    elif event.source.childCount > 0:
-		selectedItem = event.source.child(0)
-	    if selectedItem:
-		orca.setLocusOfFocus(event, selectedItem)
-	    else:
-		# if impossible to get selection or list has 0 items, present list
-		orca.setLocusOfFocus(event, event.source)
-	elif role == rolenames.ROLE_LABEL:
-	    # In FileChooserDemo, when enter in a new folder, a focus event for 
-	    # the top combo box selected item (not SHOWING item) is received.
-	    # Should this check be more specific ?
-	    #
-	    if not event.source.state.count (Accessibility.STATE_SHOWING):
-		return
-	elif role == rolenames.ROLE_MENU:
-    	    # A JMenu has always selection.nSelectedChildren > 0
+            selectedItem = None
+            selection = event.source.selection
+            if selection and selection.nSelectedChildren > 0:
+                selectedItem = atspi.Accessible.makeAccessible(
+                                        selection.getSelectedChild(0))
+            elif event.source.childCount > 0:
+                selectedItem = event.source.child(0)
+            if selectedItem:
+                orca.setLocusOfFocus(event, selectedItem)
+            else:
+                # if impossible to get selection or list has 0 items, present list
+                orca.setLocusOfFocus(event, event.source)
+        elif role == rolenames.ROLE_LABEL:
+            # In FileChooserDemo, when enter in a new folder, a focus event for
+            # the top combo box selected item (not SHOWING item) is received.
+            # Should this check be more specific ?
+            #
+            if not event.source.state.count (Accessibility.STATE_SHOWING):
+                return
+        elif role == rolenames.ROLE_MENU:
+            # A JMenu has always selection.nSelectedChildren > 0
             orca.setLocusOfFocus(event, event.source)
         else:
             default.Script.onFocus(self, event)
@@ -160,6 +173,9 @@ class Script(default.Script):
         - event: the Event
         """
 
+        self._debug("onStateChanged: %s: %s '%s' (%d,%d)" % \
+                    (event.type, event.source.role, event.source.name, event.detail1, event.detail2))
+
         # This is a workaround for a java-access-bridge bug (Bug 355011)
         # where popup menu events are not sent to Orca.
         #
@@ -183,13 +199,13 @@ class Script(default.Script):
                             if item.state.count(Accessibility.STATE_ARMED):
                                 orca.setLocusOfFocus(event, item)
                                 return
-        
 
-	# In java applications the events are comming in other order:
-	# "focus:" event comes before "state-changed:focused" event
-	#
-	if (event.type == "object:state-changed:focused"):
-	    return
+
+        # In java applications the events are comming in other order:
+        # "focus:" event comes before "state-changed:focused" event
+        #
+        if (event.type == "object:state-changed:focused"):
+            return
 
         # Hand state changes when JTree labels become expanded
         # or collapsed.
@@ -197,18 +213,18 @@ class Script(default.Script):
         if ((event.source.role == rolenames.ROLE_LABEL) and \
             (event.type == "object:state-changed:expanded")):
             orca.visualAppearanceChanged(event, event.source)
-	    return
+            return
 
-	# Present a value change in case of an focused popup menu.
-	# Fix for Swing file chooser.
-	#
-	if event.type == "object:state-changed:visible" and \
-		event.source.role == rolenames.ROLE_POPUP_MENU and \
-		event.source.parent.state.count(Accessibility.STATE_FOCUSED):
-	    orca.setLocusOfFocus(event, event.source.parent)
-	    return
-	
-	default.Script.onStateChanged(self, event)
+        # Present a value change in case of an focused popup menu.
+        # Fix for Swing file chooser.
+        #
+        if event.type == "object:state-changed:visible" and \
+                event.source.role == rolenames.ROLE_POPUP_MENU and \
+                event.source.parent.state.count(Accessibility.STATE_FOCUSED):
+            orca.setLocusOfFocus(event, event.source.parent)
+            return
+
+        default.Script.onStateChanged(self, event)
 
     def onSelectionChanged(self, event):
         """Called when an object's selection changes.
@@ -217,8 +233,11 @@ class Script(default.Script):
         - event: the Event
         """
 
-	if not event.source.state.count (atspi.Accessibility.STATE_FOCUSED):
-	    return
+        self._debug("onSelectionChanged: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
+        if not event.source.state.count (atspi.Accessibility.STATE_FOCUSED):
+            return
 
         if event.source.role == rolenames.ROLE_TABLE:
             return
@@ -261,11 +280,15 @@ class Script(default.Script):
     def onValueChanged(self, event):
         """Called whenever an object's value changes.  Currently, the
         value changes for non-focused objects are ignored.
-        
+
         Arguments:
         - event: the Event
         """
-        
+
+        self._debug("onValueChanged: %s '%s' value=%d" % \
+                    (event.source.role, event.source.name,
+                     event.source.value.currentValue))
+
         # We'll let state-changed:checked event to be used to
         # manage check boxes in java application
         #
@@ -273,6 +296,8 @@ class Script(default.Script):
                event.type == "object:property-change:accessible-value":
             return
 
+        default.Script.onValueChanged(self, event)
+
 
     def visualAppearanceChanged(self, event, obj):
         """Called when the visual appearance of an object changes.  This
@@ -287,6 +312,9 @@ class Script(default.Script):
         - obj: the Accessible whose visual appearance changed.
         """
 
+        self._debug("visualAppearanceChanged: %s '%s', locusOfFocus=%s" % \
+                    (obj.role, obj.name, orca_state.locusOfFocus.role))
+
         # Present to Braille and Speech an object that is the same
         # with the last focused object, only if it is a label.
         # This case is for LISTs which contain labels as items
@@ -298,5 +326,47 @@ class Script(default.Script):
                 speech.speakUtterances(self.speechGenerator.getSpeech(orca_state.locusOfFocus, True))
                 return
 
+        if obj.role == rolenames.ROLE_SPIN_BOX:
+            # Check for the spinbox text object being null. There appears
+            # to be a bug where the text object for some text fields
+            # is null. 
+            if orca_state.locusOfFocus.role == rolenames.ROLE_TEXT and \
+               orca_state.locusOfFocus.text == None:
+                # Set the locusOfFocus to the spinbox itself.
+                orca.setLocusOfFocus(event, obj)
+
         default.Script.visualAppearanceChanged(self, event, obj)
 
+    def onCaretMoved(self, event):
+        """Called whenever the caret moves.
+
+        Arguments:
+        - event: the Event
+        """
+
+        self._debug("onCaretMoved: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
+        # Check for bogus events containing null text objects.
+        if event.source.text == None:
+            self._debug("bogus event: text object is null. Discarding event")
+            return;
+
+        default.Script.onCaretMoved(self, event)
+
+    def onTextInserted(self, event):
+        """Called whenever the text is inserted.
+
+        Arguments:
+        - event: the Event
+        """
+
+        self._debug("onTextInserted: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
+        # Check for bogus events containing null text objects.
+        if event.source.text == None:
+            self._debug("bogus event: text object is null. Discarding event")
+            return;
+
+        default.Script.onTextInserted(self, event)        
