@@ -19,10 +19,10 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py 2047 2007-02-23 01:52:09Z lmonsanto $"
-__version__   = "$Revision: 2047 $"
-__date__      = "$Date: 2007-02-22 20:52:09 -0500 (Thu, 22 Feb 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: atspi.py 2441 2007-06-01 19:31:12Z wwalker $"
+__version__   = "$Revision: 2441 $"
+__date__      = "$Date: 2007-06-01 15:31:12 -0400 (Fri, 01 Jun 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import signal
@@ -489,13 +489,16 @@ class Accessible:
         - e: AT-SPI event from the AT-SPI registry
         """
 
-        # [[[TODO: WDW - I put this in here for now.  The idea is that
-        # we will probably get parent changed events for objects that
-        # are or will soon be defunct, so let's just forget about the
-        # object rather than try to keep the cache in sync.]]]
+        # When the parent of an object changes, just forget about the
+        # parent -- we'll lazily ask for it the next time someone is
+        # really interested.
         #
-        if Accessible._cache.has_key(e.source):
-            Accessible.deleteAccessible(e.source)
+        try:
+            obj = Accessible._cache[e.source]
+            del obj.__dict__["parent"]
+        except:
+            pass
+
         return
 
     _onParentChanged = staticmethod(_onParentChanged)
@@ -596,6 +599,12 @@ class Accessible:
         - acc: the AT-SPI Accessibility_Accessible
         """
 
+        # Just a little sanity check in case someone sent us a
+        # Python Accessible instance instead of a CORBA object.
+        #
+        if isinstance(acc, Accessible):
+            acc = acc._acc
+
         if acc and Accessible._cache.has_key(acc):
             try:
                 del Accessible._cache[acc]
