@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 2371 2007-05-11 15:28:14Z richb $"
-__version__   = "$Revision: 2371 $"
-__date__      = "$Date: 2007-05-11 11:28:14 -0400 (Fri, 11 May 2007) $"
+__id__        = "$Id: orca.py 2410 2007-05-21 18:28:50Z richb $"
+__version__   = "$Revision: 2410 $"
+__date__      = "$Date: 2007-05-21 14:28:50 -0400 (Mon, 21 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -432,7 +432,9 @@ def _isPrintableKey(event_string):
     if event_string == "space":
         reply = True
     else:
-        reply = event_string in string.printable
+        unicodeString = event_string.decode("UTF-8")
+        reply = (len(unicodeString) == 1) \
+                and (unicodeString.isalnum() or unicodeString.isspace())
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoPrintableKey: returning: %s" % reply)
     return reply
