@@ -24,9 +24,9 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py 2289 2007-04-17 15:27:48Z richb $"
-__version__   = "$Revision: 2289 $"
-__date__      = "$Date: 2007-04-17 11:27:48 -0400 (Tue, 17 Apr 2007) $"
+__id__        = "$Id: orca_gui_prefs.py 2366 2007-05-09 21:22:51Z richb $"
+__version__   = "$Revision: 2366 $"
+__date__      = "$Date: 2007-05-09 17:22:51 -0400 (Wed, 09 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -309,6 +309,14 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
 
         return voiceACSS
 
+    def writeUserPreferences(self):
+        """Write out the user's generic Orca preferences.
+        """
+
+        if orca_prefs.writePreferences(self.prefsDict, self.keyBindingsModel):
+            self._say(_("Accessibility support for GNOME has just been enabled."))
+            self._say(_("You need to log out and log back in for the change to take effect."))
+
     def _getKeyValueForVoiceType(self, voiceType, key, useDefault=True):
         """Look for the value of the given key in the voice dictionary
            for the given voice type.
@@ -976,6 +984,8 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         #
         self.showMainWindowCheckButton.set_active(prefs["showMainWindow"])
         self.confirmQuitCheckButton.set_active(prefs["quitOrcaNoConfirmation"])
+        self.presentTooltipsCheckButton.set_active( \
+            prefs["presentToolTips"] and settings.canPresentToolTips)
 
         self.disableKeyGrabPref = settings.isGKSUGrabDisabled()
         self.disableKeyGrabCheckButton.set_active(self.disableKeyGrabPref)
@@ -1235,10 +1245,15 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
-    def _populateKeyBindings(self):
+    def _populateKeyBindings(self, clearModel=True):
         """Fills the TreeView with the list of Orca keybindings
+
+        Arguments:
+        - clearModel: if True, initially clear out the key bindings model.
         """
-        self.keyBindingsModel.clear()
+
+        if clearModel:
+            self.keyBindingsModel.clear()
 
         self.kbindings = keybindings.KeyBindings()
 
@@ -2042,6 +2057,20 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
 
         self.prefsDict["quitOrcaNoConfirmation"] = widget.get_active()
 
+    def presentTooltipsChecked(self, widget):
+        """Signal handler for the "toggled" signal for the
+           presentTooltipsCheckButton GtkCheckButton widget.
+           The user has [un]checked the 'Present Tooltips'
+           checkbox. Set the 'presentToolTips'
+           preference to the new value if the user can present tooltips.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.prefsDict["presentToolTips"] = widget.get_active() and \
+                                            settings.canPresentToolTips
+
     def disableKeyGrabChecked(self, widget):
         """Signal handler for the "toggled" signal for the
            disableKeyGrabCheckButton GtkCheckButton widget.
@@ -2120,9 +2149,7 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
 
         settings.setGKSUGrabDisabled(self.disableKeyGrabPref)
 
-        if orca_prefs.writePreferences(self.prefsDict, self.keyBindingsModel):
-            self._say(_("Accessibility support for GNOME has just been enabled."))
-            self._say(_("You need to log out and log back in for the change to take effect."))
+        self.writeUserPreferences()
 
         orca.loadUserSettings()
 
