@@ -28,11 +28,11 @@ __license__   = "LGPL"
 
 # $ORCA_MAJOR_VERSION.$ORCA_MINOR_VERSION.$ORCA_MICRO_VERSION
 #
-version     = "2.19.1"
+version     = "2.19.3"
 
 # "--prefix" parameter used when configuring the build.
 #
-prefix      = "/usr"
+prefix      = "/usr/local"
 
 # The package name (should be "orca").
 #
