@@ -19,12 +19,15 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: StarOffice.py 2296 2007-04-18 16:45:21Z richb $"
-__version__   = "$Revision: 2296 $"
-__date__      = "$Date: 2007-04-18 12:45:21 -0400 (Wed, 18 Apr 2007) $"
+__id__        = "$Id: StarOffice.py 2414 2007-05-21 20:09:14Z richb $"
+__version__   = "$Revision: 2414 $"
+__date__      = "$Date: 2007-05-21 16:09:14 -0400 (Mon, 21 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import atk
+import gtk
+
 import orca.debug as debug
 import orca.atspi as atspi
 import orca.chnames as chnames
@@ -43,6 +46,10 @@ import orca.where_am_I as where_am_I
 
 from orca.orca_i18n import _ # for gettext support
 
+# Whether we speak spread sheet cell coordinates as the user moves around.
+#
+speakCellCoordinates = True
+
 class WhereAmI(where_am_I.WhereAmI):
 
     def __init__(self, script):
@@ -230,6 +237,7 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         and the second element is the Region which should get focus.
         """
 
+        focusRegion = None
         regions = []
 
         # Check to see if this spread sheet cell has either a dynamic
@@ -302,7 +310,9 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
 
                 if speakAll:
                     focusRowRegion = None
-                    for i in range(0, parent.table.nColumns):
+                    [startIndex, endIndex] = \
+                        self._script.getSpreadSheetRowRange(obj)
+                    for i in range(startIndex, endIndex+1):
                         accRow = parent.table.getAccessibleAt(row, i)
                         cell = atspi.Accessible.makeAccessible(accRow)
                         showing = cell.state.count( \
@@ -448,6 +458,33 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         return utterances
 
+    def _getSpeechForTable(self, obj, already_focused):
+        """Get the speech for a table
+
+        Arguments:
+        - obj: the table
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = self._getDefaultSpeech(obj, already_focused)
+
+        self._debugGenerator("_getSpeechForTable",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        # If this is a table with no children, then let the user know.
+        #
+        if not obj.childCount:
+            # Translators: this indicates that there are zero items in 
+            # a layered pane or table.
+            #
+            utterances.append(_("0 items"))
+
+        return utterances
+
     def _getSpeechForTableCellRow(self, obj, already_focused):
         """Get the speech for a table cell row or a single table cell
         if settings.readTableCellRow is False. If this isn't inside a
@@ -527,7 +564,9 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                                 parent.lastColumn == column)
 
                     if speakAll:
-                        for i in range(0, parent.table.nColumns):
+                        [startIndex, endIndex] = \
+                            self._script.getSpreadSheetRowRange(obj)
+                        for i in range(startIndex, endIndex+1):
                             accRow = parent.table.getAccessibleAt(row, i)
                             cell = atspi.Accessible.makeAccessible(accRow)
                             showing = cell.state.count( \
@@ -560,6 +599,8 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         Returns a list of utterances to be spoken for the object.
         """
 
+        global speakCellCoordinates
+
         if self._script.isSpreadSheetCell(obj):
             utterances = []
 
@@ -569,10 +610,17 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
             if obj.text:
                 objectText = self._script.getText(obj, 0, -1)
+                if not speakCellCoordinates and len(objectText) == 0:
+                    # Translators: this indicates an empty (blank) spread
+                    # sheet cell.
+                    #
+                    objectText = _("blank")
+
                 utterances.append(objectText)
 
-            nameList = obj.name.split()
-            utterances.append(nameList[1])
+            if speakCellCoordinates:
+                nameList = obj.name.split()
+                utterances.append(nameList[1])
         else:
             # Check to see how many children this table cell has. If it's
             # just one (or none), then pass it on to the superclass to be
@@ -719,6 +767,15 @@ class Script(default.Script):
         #
         self.tickled = None
 
+    def activate(self):
+        """Called when this script is activated."""
+        self.savedreadTableCellRow = settings.readTableCellRow
+        settings.readTableCellRow = False
+
+    def deactivate(self):
+        """Called when this script is deactivated."""
+        settings.readTableCellRow = self.savedreadTableCellRow
+
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
@@ -802,6 +859,86 @@ class Script(default.Script):
 
         return keyBindings
 
+    def getAppPreferencesGUI(self):
+        """Return a GtkVBox contain the application unique configuration
+        GUI items for the current application.
+        """
+
+        global speakCellCoordinates
+
+        vbox = gtk.VBox(False, 0)
+        vbox.set_border_width(12)
+        gtk.Widget.show(vbox)
+
+        # Text entry and label for "Enabled Text Attributes:".
+        #
+        hbox = gtk.HBox(False, 0)
+        gtk.Widget.show(hbox)
+
+        # Translators: All text in StarOffice/OpenOffice has a set of
+        # attributes associated with it (such as font type, size, style...)
+        # This label denotes those attributes that you user would like
+        # spoken when you ask to hear the text attributes associated with
+        # the current text (typically when the Insert-f key combination is 
+        # pressed).
+        #
+        self.textAttributesLabel = gtk.Label(_("Enabled Text Attributes:"))
+        gtk.Widget.show(self.textAttributesLabel)
+        gtk.Box.pack_start(hbox, self.textAttributesLabel, False, False, 5)
+
+        self.enabledTextAttributes = gtk.Entry()
+        gtk.Widget.show(self.enabledTextAttributes)
+        self.enabledTextAttributes.set_text(orca.settings.enabledTextAttributes)
+        gtk.Box.pack_end(hbox, self.enabledTextAttributes, True, True, 5)
+
+        acc_targets = []
+        acc_src = self.textAttributesLabel.get_accessible()
+        relation_set = acc_src.ref_relation_set()
+        acc_targ = self.enabledTextAttributes.get_accessible()
+        acc_targets.append(acc_targ)
+        relation = atk.Relation(acc_targets, 1)
+        relation.set_property('relation-type', atk.RELATION_LABEL_FOR)
+        relation_set.add(relation)
+
+        gtk.Box.pack_start(vbox, hbox, False, False, 0)
+
+        # Checkbox for "Speak spread sheet cell coordinates".
+        #
+        # Translators: If checked, then Orca will speak the coordinates
+        # of the current spread sheet cell. Coordinates are the row and
+        # column position within the spread sheet (i.e. A1, B1, C2 ...)
+        #
+        label = _("Speak spread sheet cell coordinates")
+        self.speakCellCoordinatesCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.speakCellCoordinatesCheckButton)
+        gtk.Box.pack_start(vbox, self.speakCellCoordinatesCheckButton, 
+                           False, False, 0)
+        gtk.ToggleButton.set_active(self.speakCellCoordinatesCheckButton,
+                                    speakCellCoordinates)
+
+        return vbox
+
+    def setAppPreferences(self, prefs):
+        """Write out the application specific preferences lines and set the
+        new values.
+
+        Arguments:
+        - prefs: file handle for application preferences.
+        """
+
+        global speakCellCoordinates
+
+        settings.enabledTextAttributes = self.enabledTextAttributes.get_text()
+        prefs.writelines("\n")
+        prefs.writelines("orca.settings.enabledTextAttributes = '%s'\n" %  \
+                         settings.enabledTextAttributes)
+
+        speakCellCoordinates = \
+                 self.speakCellCoordinatesCheckButton.get_active()
+        prefs.writelines( \
+                 "orca.scripts.StarOffice.speakCellCoordinates = %s\n" % \
+                 speakCellCoordinates)
+
     def adjustForWriterTable(self, obj):
         """Check to see if we are in Writer, where the object with focus
         is a paragraph, and the parent is the table cell. If it is, then,
@@ -910,6 +1047,39 @@ class Script(default.Script):
 
         return inputLine
 
+    def getSpreadSheetRowRange(self, obj):
+        """If this is spread sheet cell, return the start and end indices 
+        of the spread sheet cells for the table that obj is in. Otherwise
+        return the complete range (0, parent.table.nColumns).
+
+        Arguments:
+        - obj: a spread sheet table cell.
+
+        Returns the start and end table cell indices.
+        """
+
+        parent = obj.parent
+        startIndex = 0
+        endIndex = parent.table.nColumns
+
+        if self.isSpreadSheetCell(obj):
+            y = parent.extents.y
+            leftX = parent.extents.x + 1
+            cell = parent.component.getAccessibleAtPoint(leftX, y, 0)
+            if cell:
+                leftCell = atspi.Accessible.makeAccessible(cell)
+                table = leftCell.parent.table
+                startIndex = table.getColumnAtIndex(leftCell.index)
+
+            rightX = parent.extents.x + parent.extents.width - 1
+            cell = parent.component.getAccessibleAtPoint(rightX, y, 0)
+            if cell:
+                rightCell = atspi.Accessible.makeAccessible(cell)
+                table = rightCell.parent.table
+                endIndex = table.getColumnAtIndex(rightCell.index)
+
+        return [startIndex, endIndex]
+
     def isSpreadSheetCell(self, obj):
         """Return an indication of whether the given obj is a spread sheet
         table cell.
@@ -1411,7 +1581,8 @@ class Script(default.Script):
             # keying off stuff like this, but we're forced to do so in
             # this case.
             #
-            if oldLocusOfFocus.role == rolenames.ROLE_MENU_ITEM and \
+            if oldLocusOfFocus and \
+               oldLocusOfFocus.role == rolenames.ROLE_MENU_ITEM and \
                oldLocusOfFocus.name == _("Text Document") and \
                len(result[0]) == 0:
                 self.whereAmI(None)
@@ -1796,7 +1967,8 @@ class Script(default.Script):
             # See bug #363830 and OOo issue #70872.]]]
             #
             if not self.tickled:
-                self.tickled = self.flatReviewContextClass(self)
+                self.getObjects(self.getFrame(event.source))
+                self.tickled = True
 
         # Announce when the toolbar buttons are toggled if we just toggled
         # them; not if we navigated to some text.
@@ -1812,7 +1984,7 @@ class Script(default.Script):
                 weToggledIt = event.source.component.contains(x, y, 0)
 
             elif isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
-                keyString = orca_state.lastInputEvent.event_string
+                keyString = orca_state.lastNonModifierKeyEvent.event_string
                 navKeys = ["Up", "Down", "Page_Up", "Page_Down", "Home", "End"]
                 wasCommand = orca_state.lastInputEvent.modifiers \
                              & (1 << atspi.Accessibility.MODIFIER_CONTROL \
@@ -1843,7 +2015,7 @@ class Script(default.Script):
             #
             event_string = None
             if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
-                event_string = orca_state.lastInputEvent.event_string
+                event_string = orca_state.lastNonModifierKeyEvent.event_string
             if (event_string == "Up" or event_string == "Down") and \
                event.source.parent == self.lastCell and \
                event.source.parent.childCount > 1:
@@ -2003,7 +2175,7 @@ class Script(default.Script):
         if event.source.role == rolenames.ROLE_PARAGRAPH and \
            event.source.parent.role == rolenames.ROLE_TABLE_CELL and \
            event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
-            event_string = orca_state.lastInputEvent.event_string
+            event_string = orca_state.lastNonModifierKeyEvent.event_string
 
             # If we are moving up and down, and we are speaking-by-cell
             # (as opposed to by-row), then speak the cell name. Otherwise
@@ -2084,7 +2256,7 @@ class Script(default.Script):
                 orca_state.lastInputEvent.__dict__.has_key("event_string")):
             return False
 
-        lastKey = orca_state.lastInputEvent.event_string
+        lastKey = orca_state.lastNonModifierKeyEvent.event_string
         if lastKey != "Left" and lastKey != "Right":
             return False
 
