@@ -19,9 +19,9 @@
 
 """Custom script for gedit."""
 
-__id__        = "$Id: gedit.py 2289 2007-04-17 15:27:48Z richb $"
-__version__   = "$Revision: 2289 $"
-__date__      = "$Date: 2007-04-17 11:27:48 -0400 (Tue, 17 Apr 2007) $"
+__id__        = "$Id: gedit.py 2371 2007-05-11 15:28:14Z richb $"
+__version__   = "$Revision: 2371 $"
+__date__      = "$Date: 2007-05-11 11:28:14 -0400 (Fri, 11 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -509,7 +509,7 @@ class Script(default.Script):
         #
         if event.source.role == rolenames.ROLE_STATUSBAR \
            and self.isFocusOnFindDialog() \
-           and orca_state.lastInputEvent.event_string == "Return" \
+           and orca_state.lastNonModifierKeyEvent.event_string == "Return" \
            and event.source.name == _("Phrase not found"):
                 debug.println(self.debugLevel,
                               "gedit.onNameChanged - phrase not found.")
@@ -564,7 +564,7 @@ class Script(default.Script):
         # we've just found.
         #
         if self.isFocusOnFindDialog() \
-           and orca_state.lastInputEvent.event_string == "Return":
+           and orca_state.lastNonModifierKeyEvent.event_string == "Return":
             debug.println(self.debugLevel, "gedit.onCaretMoved - find dialog.")
 
             allComboBoxes = self.findByRole(orca_state.locusOfFocus.app,
